# 한글-영어 변환
KOR_WORDS = list('ㄱㄲㄳㄴㄵㄶㄷㄹㄺㄻㄼㄽㄾㄿㅀㅁㅂㅄㅅㅆㅇㅈㅊㅋㅌㅍㅎㅏㅐㅑㅒㅓㅔㅕㅖㅗㅘㅙㅚㅛㅜㅝㅞㅟㅠㅡㅢㅣ')
ENG_WORDS = [
    'r', 'R', 'rt', 's', 'sw', 'sg', 'e', 'f', 'fr', 'fa', 'fq', 'ft',
    'fx', 'fv', 'fg', 'a', 'q', 'qt', 't', 'T', 'd', 'w', 'c', 'z',
    'x', 'v', 'g', 'k', 'o', 'i', 'O', 'j', 'p', 'u', 'P', 'h', 'hk',
    'ho', 'hl', 'y', 'n', 'nj', 'np', 'nl', 'b', 'm', 'ml', 'l'
]
KOR_ENG_TABLE = dict(zip(KOR_WORDS, ENG_WORDS))



import re
from hangul_utils import split_syllables
from korea_public_data.core.consts.util import KOR_ENG_TABLE


def dict_to_str(data: dict) -> str:
    """딕셔너리를 모두 연결"""
    return "".join([f"{key}{value}" for key, value in data.items()])


def kor_text_to_id(kor_text: str) -> str:
    """일반 텍스트를 id로 변환"""
    sub_kor_text = re.sub('[^A-Za-z0-9가-힣]', '', kor_text).strip()
    split_text = split_syllables(sub_kor_text)
    return ''.join(
        [
            word.replace(word, KOR_ENG_TABLE[word])
            if KOR_ENG_TABLE.get(word) else word
            for word in str(split_text)
        ]
    )



from os import environ

from korea_public_data.service.app import PublicDataController


def test_exchange():
    """환율 조회 API"""
    # 앱 실행
    auth_key = environ.get("KOREA_BANK_AUTH_KEY")
    app_name = 'exchangeJSON'

    if auth_key:
        controller = PublicDataController(app_name).set_keys(
            auth_key=auth_key
        )
        assert type(controller.data) == list



from collections import OrderedDict
from os import environ

from korea_public_data.service.app import PublicDataController


def create_app_and_status_check(app_name, service_key, **kwargs):
    controller = PublicDataController(app_name).set_keys(
        service_key=service_key, **kwargs
    )

    # 데이터 검증
    if service_key:
        res = controller.data
        assert isinstance(controller.data, OrderedDict)
        assert res.get('response').get('header').get('resultCode') in ['00', '99']
        assert res.get('response').get('header').get('resultMsg') in [
            'NORMAL SERVICE.', 'SERVICE KEY IS NOT REGISTERED ERROR.'
        ]

        return res
    else:
        try:
            controller.data
        except Exception as ex:
            assert str(ex) == "서비스 키가 등록되지 않았습니다."


def test_request_apt_trade_api():
    """아파트 매매 API"""
    # 앱 실행
    service_key = environ.get("DATA_GO_KR_SERVICE_KEY")
    app_name = 'getRTMSDataSvcAptTrade'
    res = create_app_and_status_check(app_name, service_key)


def test_request_apt_trade_dev_api():
    """아파트 매매 상세 API"""
    # 앱 실행
    service_key = environ.get("DATA_GO_KR_SERVICE_KEY")
    app_name = 'getRTMSDataSvcAptTradeDev'
    res = create_app_and_status_check(app_name, service_key)


def test_request_covid():
    """코로나 감염 현황 API"""
    # 앱 실행
    service_key = environ.get("DATA_GO_KR_SERVICE_KEY")
    app_name = 'getCovid19InfStateJson'
    res = create_app_and_status_check(app_name, service_key)


def test_request_business_man():
    """사업자 등록정보 진위확인 및 상태조회 서비스 API"""
    # 앱 실행
    service_key = environ.get("DATA_GO_KR_SERVICE_KEY")
    app_name = 'nTSBusinessMan'

    if service_key:
        # 기본 사업자 조회
        controller = PublicDataController(app_name).set_keys(
            service_key=service_key
        )
        assert controller.data.get("request_cnt") == 1
        assert controller.data.get("data")[0].get("b_no") == "1208147521"

        # 변경된 사업자 조회
        controller.business_no = ["2208162517"]
        assert controller.data.get("request_cnt") == 1
        assert controller.data.get("data")[0].get("b_no") == "2208162517"


def test_station_by_route():
    """서울 버스 노선 정보 API"""
    # 앱 실행
    service_key = environ.get("DATA_GO_KR_SERVICE_KEY")
    app_name = 'getStationByRoute'

    if service_key:
        controller = PublicDataController(app_name).set_keys(
            service_key=service_key
        )
        assert isinstance(controller.data, OrderedDict)



APT_TRADE_URL = "http://openapi.molit.go.kr:8081/OpenAPI_ToolInstallPackage/service/rest/RTMSOBJSvc/getRTMSDataSvcAptTrade?"
APT_TRADE_DEV_URL = "http://openapi.molit.go.kr/OpenAPI_ToolInstallPackage/service/rest/RTMSOBJSvc/getRTMSDataSvcAptTradeDev?"
COVID_INFECTION_STATUS_URL = "http://openapi.data.go.kr/openapi/service/rest/Covid19/getCovid19InfStateJson?"
NTS_BUSINESS_MAN_URL = "https://api.odcloud.kr/api/nts-businessman/v1/status?"
STATION_BY_ROUTE_URL = "http://ws.bus.go.kr/api/rest/busRouteInfo/getStaionByRoute?"
BANKING_EXCHANGE_URL = "https://www.koreaexim.go.kr/site/program/financial/exchangeJSON?"


import json
import xmltodict
import requests
from abc import ABCMeta, abstractmethod
from korea_public_data.core.choices import ResponseType


class PublicDataBase(metaclass=ABCMeta):
    def __init__(self):
        self.response_type = None
        self.validated_data = {}

    @property
    @abstractmethod
    def url(self):
        return None

    @abstractmethod
    def _data_valid(self):
        pass

    def _additional_processing(self, res):
        """상태에 따른 분류"""
        pass

    def _valid(self):
        """데이터 유효성 검사"""
        # 기본 유효성 검사
        assert self.url, "API URL 이 적용되지 않았습니다."
        assert self.response_type, "반환 값 종류가 적용되지 않았습니다."

        # 입력한 데이터 확인
        self._data_valid()

        # API 호출 오류 확인
        res = self._get_data()
        res.raise_for_status()

        # 데이터 입력
        self.validated_data['res'] = res

    def _get_data(self):
        """데이터 조회"""
        return requests.get(self.url)

    @staticmethod
    def json_to_dict(json_response: str):
        """JSON 딕셔너리로 전환"""
        return json.loads(json_response)

    @staticmethod
    def xml_to_dict(xml_response: str):
        """XML 딕셔너리로 전환"""
        return xmltodict.parse(xml_response)

    def _convert_response(self, res):
        """결과 타입에 따라 데이터 변환"""
        if self.response_type == ResponseType.XML:
            res = self.xml_to_dict(res.content)
        elif self.response_type == ResponseType.JSON:
            res = self.json_to_dict(res.content)
        return res

    @property
    def data(self):
        """데이터 결과 조회"""
        # 유효성 검사
        self._valid()

        # 데이터 조회
        res = self.validated_data.get('res')
        if res is None:
            raise ValueError("결과 값이 포함되어있지 않습니다.")

        # 상태에 따른 추가 처리 (오버라이딩 권장)
        self._additional_processing(res)

        # 결과 값 반환
        return self._convert_response(res)



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class GetCovid19InfStateJson(PublicDataBase):
    """공공데이터활용지원센터_보건복지부 코로나19 감염 현황"""
    def __init__(self, service_key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.service_key = service_key
        # 변수(기본 값 적용)
        self.yesterday = var.DEFAULT_YESTERDAY_SEOUL_TIMEZONE
        self.start_at = self.yesterday.strftime("%Y%m%d")
        self.end_at = self.yesterday.strftime("%Y%m%d")
        self.page_no = var.DEFAULT_PAGE_NO
        self.num_of_rows = var.DEFAULT_PAGE_NUM_OF_ROWS
        # 상수
        self.response_type = ResponseType.XML

    @property
    def url(self):
        return (
            f'{const.COVID_INFECTION_STATUS_URL}'
            f'startCreateDt={self.start_at}&'
            f'endCreateDt={self.end_at}&'
            f'pageNo={self.page_no}&'
            f'numOfRows={self.num_of_rows}&'
            f'serviceKey={self.service_key}'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.service_key, "서비스 키가 등록되지 않았습니다."



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class GetRTMSDataSvcAptTrade(PublicDataBase):
    """국토교통부_아파트매매 실거래자료"""
    def __init__(self, service_key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.service_key = service_key
        # 변수(기본 값 적용)
        self.location_code = var.DEFAULT_APT_TRADE_LOCATION_CODE
        self.yesterday = var.DEFAULT_YESTERDAY_SEOUL_TIMEZONE
        self.search_date = self.yesterday.strftime("%Y%m")
        # 상수
        self.response_type = ResponseType.XML

    @property
    def url(self):
        return (
            f'{const.APT_TRADE_URL}'
            f'LAWD_CD={self.location_code}&'
            f'DEAL_YMD={self.search_date}&'
            f'serviceKey={self.service_key}'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.service_key, "서비스 키가 등록되지 않았습니다."



from korea_public_data.data.data_go_kr.apt_trade import GetRTMSDataSvcAptTrade
from korea_public_data.data.data_go_kr.apt_trade_detail import GetRTMSDataSvcAptTradeDev
from korea_public_data.data.data_go_kr.covid_infection_status import GetCovid19InfStateJson
from korea_public_data.data.data_go_kr.nts_businessman import NTSBusinessMan
from korea_public_data.data.data_go_kr.station_by_route import GetStationByRoute
from korea_public_data.data.koreaexim_go_kr.exchange import ExchangeJSON

DATA_CLASSES = {
    "getRTMSDataSvcAptTrade": GetRTMSDataSvcAptTrade,
    "getRTMSDataSvcAptTradeDev": GetRTMSDataSvcAptTradeDev,
    "getCovid19InfStateJson": GetCovid19InfStateJson,
    "nTSBusinessMan": NTSBusinessMan,
    "getStationByRoute": GetStationByRoute,
    "exchangeJSON": ExchangeJSON
}



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class GetStationByRoute(PublicDataBase):
    """서울특별시_노선정보조회 서비스"""
    def __init__(self, service_key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.service_key = service_key
        # 변수(기본 값 적용)
        self.bus_route_id = var.DEFAULT_STATION_BY_ROUTE_ID
        # 상수
        self.response_type = ResponseType.XML

    @property
    def url(self):
        return (
            f'{const.STATION_BY_ROUTE_URL}'
            f'serviceKey={self.service_key}&'
            f'busRouteId={self.bus_route_id}'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.service_key, "서비스 키가 등록되지 않았습니다."



from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class ExchangeJSON(PublicDataBase):
    """현재 환율 API"""
    def __init__(self, auth_key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.auth_key = auth_key
        # 변수(기본 값 적용)
        self.yesterday = var.DEFAULT_TODAY_SEOUL_TIMEZONE
        self.search_date = self.yesterday.strftime("%Y%m%d")
        # 상수
        self.response_type = ResponseType.JSON

    @property
    def url(self):
        return (
            f'{const.BANKING_EXCHANGE_URL}'
            f'authkey={self.auth_key}&'
            f'searchdate={self.search_date}&'
            f'data=AP01'
        )

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.auth_key, "서비스 키가 등록되지 않았습니다."



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="korea-public-data",  # Replace with your own PyPI username(id)
    version="0.0.2",
    author="JAY",
    author_email="root@ja-y.com",
    description="Korea Public Data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ja-y-com/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "certifi==2021.10.8",
        "charset-normalizer==2.0.12",
        "hangul-utils==0.2",
        "idna==3.3",
        "pytz==2022.1",
        "requests==2.27.1",
        "urllib3==1.26.9",
        "xmltodict==0.12.0"
    ]
)



from datetime import datetime, timedelta

import pytz as pytz


DEFAULT_APT_TRADE_LOCATION_CODE = "00440"
DEFAULT_YESTERDAY_SEOUL_TIMEZONE = datetime.now().astimezone(pytz.timezone("Asia/Seoul")) - timedelta(days=1)
DEFAULT_TODAY_SEOUL_TIMEZONE = datetime.now().astimezone(pytz.timezone("Asia/Seoul"))
DEFAULT_PAGE_NO = 1
DEFAULT_PAGE_NUM_OF_ROWS = 100
DEFAULT_STATION_BY_ROUTE_ID = "14194"
DEFAULT_BUSINESS_MAN_NUMBER = ["1208147521"]



from korea_public_data.core.utils.convert import kor_text_to_id, dict_to_str


def test_create_id():
    # 변환 데이터
    dict_values = {
        "지역": "서울",
        "일시": "2022-01-01 10:00",
        "확률": "50%"
    }
    str_value = dict_to_str(dict_values)
    idx = kor_text_to_id(str_value)

    # 검증 데이터
    location = "wldurtjdnf"  # 지역
    datetime = "dlftl202201011000"  # 일시
    percent = "ghkrfbf50"  # 확률
    assert idx == f"{location}{datetime}{percent}"



import json
import requests

from korea_public_data.core.choices import ResponseType
from korea_public_data.core.vars import default as var
from korea_public_data.core.consts import data as const
from korea_public_data.data.base import PublicDataBase


class NTSBusinessMan(PublicDataBase):
    """국세청_사업자등록정보 진위확인 및 상태조회 서비스"""
    def __init__(self, service_key: str):
        # 상위 클래스 변수 적용
        super().__init__()
        # 필수 설정
        self.service_key = service_key
        self.business_no = var.DEFAULT_BUSINESS_MAN_NUMBER
        # 상수
        self.response_type = ResponseType.JSON

    @property
    def url(self):
        return (
            f'{const.NTS_BUSINESS_MAN_URL}'
            f'serviceKey={self.service_key}'
        )

    def _get_data(self):
        """데이터 조회"""
        headers = {"Content-Type": "application/json"}
        data = {"b_no": self.business_no}
        return requests.post(self.url, data=json.dumps(data), headers=headers)

    def _data_valid(self):
        """데이터 이상여부 확인"""
        assert self.service_key, "서비스 키가 등록되지 않았습니다."
        assert self.business_no, "조회할 사업자 번호가 필요합니다."
        assert type(self.business_no) == list, "리스트의 형태만 검색이 가능합니다."


from korea_public_data.core.vars.data import DATA_CLASSES


class PublicDataController:
    def __init__(
            self,
            app: str,  # 앱 이름
    ):
        self._data_class = DATA_CLASSES.get(app)
        assert self._data_class, "선택하신 앱이 존재하지 않습니다."

    def set_keys(self, *args, **kwargs):
        """각 클래스의 키 값을 입력"""
        return self._data_class(*args, **kwargs)
